@@ -19,17 +19,14 @@
 #ifndef SEVENZIP_H
 #define SEVENZIP_H
 
-#include <gtk/gtk.h>
-#include <stdlib.h>
-#include "string_utils.h"
-#include "support.h"
+#include <glib.h>
 #include "archive.h"
 
-gboolean jump_header, last_line;
-void xa_7zip_delete (XArchive *,GSList *);
-void xa_7zip_add (XArchive *,GString *,gchar *);
-gboolean xa_7zip_extract(XArchive *,GSList *);
-void xa_7zip_test (XArchive *);
-void xa_open_7zip (XArchive *);
-void xa_get_7zip_line_content (gchar *, gpointer );
+void xa_7zip_add(XArchive *, GSList *, gchar *);
+void xa_7zip_ask(XArchive *);
+void xa_7zip_delete(XArchive *, GSList *);
+gboolean xa_7zip_extract(XArchive *, GSList *);
+void xa_7zip_list(XArchive *);
+void xa_7zip_test(XArchive *);
+
 #endif
