@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2006 Giuseppe Torelli - <colossus73@gmail.com>
+ *  Copyright (C) 2008 Giuseppe Torelli - <colossus73@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,25 +16,33 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef MAIN_H
-#define MAIN_H
+#ifndef XARCHIVER_MAIN_H
+#define XARCHIVER_MAIN_H
 
-#include <gtk/gtk.h>
 #include <glib.h>
-#include "interface.h"
-#include "new_dialog.h"
 #include "archive.h"
-#include "callbacks.h"
+#include "gzip_et_al.h"
 
-GtkWidget *MainWindow;
-GList *ArchiveSuffix;
-GList *ArchiveType;
-const gchar *locale;
+#define COPYRIGHT_YEAR "2005-2014"
+#define COPYRIGHT_HOLDER "Giuseppe Torelli"
+#define MAINTAINER "Ingo Brückl"
 
-void GetAvailableCompressors();
-void xa_set_button_state ( gboolean, gboolean,gboolean,gboolean, gboolean, gboolean );
-gchar *get_argv_filename(const gchar *filename);
-gboolean SpawnSyncCommand ( gchar *command );
-XArchive *xa_init_structure_from_cmd_line (char *filename);
-#endif /* MAIN_H */
+extern GtkWidget *xa_main_window;
 
+extern XArchiver archiver[];
+extern const gchar *locale;
+extern gchar *xdg_open;
+
+extern add_func add[];
+extern ask_func ask[];
+extern delete_func delete[];
+extern extract_func extract[];
+extern list_func list[];
+extern test_func test[];
+
+extern struct Add_dialog_data *add_window;
+extern struct Extract_dialog_data *extract_window;
+extern struct Multi_extract_data *multi_extract_window;
+extern struct Prefs_dialog_data *prefs_window;
+
+#endif
