@@ -16,45 +16,134 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-#include <glib.h>
+#include <string.h>
 #include <gtk/gtk.h>
 #include "new_dialog.h"
-#include "window.h"
-#include "string_utils.h"
+#include "interface.h"
 #include "main.h"
+#include "pref_dialog.h"
+#include "string_utils.h"
+#include "support.h"
+#include "window.h"
 
-extern gboolean unrar;
-extern Prefs_dialog_data *prefs_window;
-gchar *current_new_directory = NULL;
-gint  new_combo_box = -1;
-gchar *ComboArchiveType;
+static GtkWidget *create;
+static gint new_combo_box = -1;
+static gchar *ComboArchiveType;
+static gchar *current_new_directory;
+
+static gpointer xa_get_suffix (const gchar *type)
+{
+	int i;
+
+	if (!type) return "";
+
+	for (i = XARCHIVETYPE_FIRST; i < XARCHIVETYPE_TYPES; i++)
+	{
+		GSList *list = archiver[i].type;
+		GSList *glob = archiver[i].glob;
+
+		while (list && glob)
+		{
+			if (list->data && (strcmp(type, list->data) == 0))
+				return glob->data + 1;
+
+			list = list->next;
+			glob = glob->next;
+		}
+	}
+
+	return "";
+}
+
+static void xa_change_archive_extension (GtkComboBox *combo_box, GtkWidget *xa_file_chooser)
+{
+	gpointer suffix;
+	gchar *fname, *file = NULL, *file_suffix = NULL, *stem, *newfile, *newfile_utf8;
+	size_t longest = 0;
+	int i;
+
+	g_free(ComboArchiveType);
+	ComboArchiveType = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(GTK_WIDGET(combo_box)));
+	suffix = xa_get_suffix(ComboArchiveType);
+
+	gtk_widget_set_sensitive(create, *(char *) suffix != 0);
+
+	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(xa_file_chooser));
+
+	if (fname)
+	{
+		file = g_path_get_basename(fname);
+		g_free(fname);
+	}
+
+	if (!file)
+		file = g_strdup("");
+
+	for (i = XARCHIVETYPE_FIRST; i < XARCHIVETYPE_TYPES; i++)
+	{
+		GSList *list = archiver[i].glob;
+
+		while (list)
+		{
+			if (*(char *) list->data == '*')
+			{
+				if (g_str_has_suffix(file, list->data + 1))
+				{
+					size_t l = strlen(list->data + 1);
+
+					if (l > longest)
+					{
+						longest = l;
+						g_free(file_suffix);
+						file_suffix = g_strdup(list->data + 1);
+					}
+				}
+			}
+
+			list = list->next;
+		}
+	}
+
+	if (file_suffix)
+		stem = g_strndup(file, strlen(file) - longest);
+	else
+		stem = g_strdup(file);
+
+	newfile = g_strconcat(stem, suffix, NULL);
+	newfile_utf8 = g_filename_display_name(newfile);
+
+	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(xa_file_chooser), newfile_utf8);
+
+	g_free(newfile_utf8);
+	g_free(newfile);
+	g_free(stem);
+	g_free(file_suffix);
+	g_free(file);
+}
 
 XArchive *xa_new_archive_dialog (gchar *path, XArchive *archive_open[], gboolean flag)
 {
+	GTK_COMPAT_TOOLTIPS;
 	XArchive *archive = NULL;
 	GtkWidget *xa_file_chooser;
 	GtkWidget *hbox = NULL;
 	GtkWidget *combo_box = NULL;
-	GtkWidget *add_extension_cb = NULL;
 	GtkFileFilter *xa_new_archive_dialog_filter;
-	GtkTooltips *filter_tooltip;
-	GList *Suffix,*Name;
+	GSList *sorted;
 	gchar *my_path = NULL;
-	gchar *my_path_ext = NULL;
-	gchar *basepath = NULL;
+	gchar *basepath, *filename;
 	gchar *current_dir = NULL;
-	gint current_page, response,type = 0;
-	unsigned short int x;
+	gint current_page, response, type = XARCHIVETYPE_UNKNOWN;
+	gpointer suffix;
+	int i;
 
 	xa_file_chooser = gtk_file_chooser_dialog_new ( _("Create a new archive"),
 							GTK_WINDOW (xa_main_window),
 							GTK_FILE_CHOOSER_ACTION_SAVE,
 							GTK_STOCK_CANCEL,
 							GTK_RESPONSE_CANCEL,
-							_("Cr_eate"),
-							GTK_RESPONSE_ACCEPT,
 							NULL);
+	create = gtk_dialog_add_button(GTK_DIALOG(xa_file_chooser), _("Cr_eate"), GTK_RESPONSE_ACCEPT);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (xa_file_chooser), GTK_RESPONSE_ACCEPT);
 
@@ -65,60 +154,47 @@ XArchive *xa_new_archive_dialog (gchar *path, XArchive *archive_open[], gboolean
 
 	xa_new_archive_dialog_filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name ( xa_new_archive_dialog_filter , _("Only archives") );
-
-	Suffix = g_list_first ( ArchiveSuffix );
-
-	while ( Suffix != NULL )
-	{
-		gtk_file_filter_add_pattern (xa_new_archive_dialog_filter, Suffix->data);
-		Suffix = g_list_next ( Suffix );
-	}
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (xa_file_chooser), xa_new_archive_dialog_filter);
 
-	Suffix = g_list_first ( ArchiveSuffix );
+	sorted = xa_file_filter_add_archiver_pattern_sort(xa_new_archive_dialog_filter);
 
-	while ( Suffix != NULL )
+	while (sorted)
 	{
-		if ( strcmp(Suffix->data, "") != 0 )	/* To avoid double filtering when opening the archive */
-		{
-			xa_new_archive_dialog_filter = gtk_file_filter_new ();
-			gtk_file_filter_set_name (xa_new_archive_dialog_filter, Suffix->data );
-			gtk_file_filter_add_pattern (xa_new_archive_dialog_filter, Suffix->data );
-			gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (xa_file_chooser), xa_new_archive_dialog_filter);
-		}
-		Suffix = g_list_next ( Suffix );
+		xa_new_archive_dialog_filter = gtk_file_filter_new();
+		gtk_file_filter_set_name(xa_new_archive_dialog_filter, sorted->data);
+		gtk_file_filter_add_pattern(xa_new_archive_dialog_filter, sorted->data);
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(xa_file_chooser), xa_new_archive_dialog_filter);
+		sorted = sorted->next;
 	}
+
+	g_slist_free(sorted);
+
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (hbox),gtk_label_new (_("Archive type:")),FALSE,FALSE,0);
 
-	combo_box = gtk_combo_box_new_text ();
+	combo_box = gtk_combo_box_text_new();
 
-	filter_tooltip = gtk_tooltips_new();
-	gtk_tooltips_set_tip (filter_tooltip,combo_box, _("Choose the archive type to create") , NULL);
-	Name = g_list_first (ArchiveType);
+	gtk_widget_set_tooltip_text(combo_box, _("Choose the archive type to create"));
+	xa_combo_box_text_append_compressor_types(GTK_COMBO_BOX_TEXT(combo_box));
 
-	while (Name)
-	{
-		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),Name->data);
-		Name = g_list_next (Name);
-	}
 	if (new_combo_box == -1)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_prefered_format)));
 	else
 		gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box),new_combo_box);
 
-	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), combo_box, FALSE, FALSE, 0);
 
-	add_extension_cb = gtk_check_button_new_with_label (_("Add the archive extension to the filename"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_extension_cb),TRUE);
-	gtk_box_pack_start (GTK_BOX (hbox), add_extension_cb, TRUE, TRUE, 0);
 	gtk_widget_show_all (hbox);
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (xa_file_chooser), hbox);
+	ComboArchiveType = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(combo_box));
+	suffix = xa_get_suffix(ComboArchiveType);
+	gtk_widget_set_sensitive(create, *(char *) suffix != 0);
+	g_signal_connect(G_OBJECT(combo_box), "changed", G_CALLBACK(xa_change_archive_extension), xa_file_chooser);
 
 	if (path != NULL)
 	{
 		basepath = g_path_get_basename (path);
-		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (xa_file_chooser),basepath);
+		filename = g_strconcat(basepath, suffix, NULL);
 
 		current_dir = g_get_current_dir ();
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (xa_file_chooser),current_dir);
@@ -126,35 +202,37 @@ XArchive *xa_new_archive_dialog (gchar *path, XArchive *archive_open[], gboolean
 		g_free (basepath);
 		g_free (current_dir);
 	}
+	else
+	{
+		filename = g_strdup(suffix);
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(xa_file_chooser), ".");
+	}
+
+	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(xa_file_chooser), filename);
+	g_free(filename);
+
 	gtk_window_set_modal (GTK_WINDOW (xa_file_chooser),TRUE);
 	if (current_new_directory != NULL)
 		gtk_file_chooser_set_current_folder ( GTK_FILE_CHOOSER (xa_file_chooser),current_new_directory);
 	response = gtk_dialog_run (GTK_DIALOG (xa_file_chooser));
 	current_new_directory = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (xa_file_chooser));
 
-	if (response == GTK_RESPONSE_ACCEPT)
+	if (!ComboArchiveType || (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT))
+		gtk_widget_destroy(xa_file_chooser);
+	else if (response == GTK_RESPONSE_ACCEPT)
 	{
 		my_path = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER (xa_file_chooser) );
-		ComboArchiveType = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo_box));
-
-		if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (add_extension_cb)))
-		{
-			if ( ! g_str_has_suffix (my_path,ComboArchiveType))
-			{
-				my_path_ext = g_strconcat (my_path, ".",ComboArchiveType,NULL);
-				g_free (my_path);
-				my_path = my_path_ext;
-			}
-		}
 
 		if (xa_main_window)
 		{
+			int x;
+
 			for (x = 0; x < gtk_notebook_get_n_pages (notebook) ; x++)
 			{
 				current_page = xa_find_archive_index (x);
 				if (current_page == -1)
 					break;
-				if (strcmp (my_path,archive_open[current_page]->path) == 0)
+				if (strcmp(my_path, archive_open[current_page]->path[0]) == 0)
 				{
 					gchar *msg = g_strdup_printf(_("\"%s\" is already open!"),my_path);
 					response = xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't create a new archive:"),msg );
@@ -194,47 +272,37 @@ XArchive *xa_new_archive_dialog (gchar *path, XArchive *archive_open[], gboolean
 
 		new_combo_box = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
 
-		if (strcmp ( ComboArchiveType,"arj") == 0)
-			type = XARCHIVETYPE_ARJ;
-		else if (strcmp ( ComboArchiveType,"bz2") == 0)
-			type = XARCHIVETYPE_BZIP2;
-		else if (strcmp ( ComboArchiveType,"gz") == 0)
-			type = XARCHIVETYPE_GZIP;
-		else if (strcmp ( ComboArchiveType,"lzma") == 0)
-			type = XARCHIVETYPE_LZMA;
-		else if (strcmp ( ComboArchiveType,"lzo") == 0)
-			type = XARCHIVETYPE_LZOP;
-		else if (strcmp ( ComboArchiveType,"rar") == 0)
-			type = XARCHIVETYPE_RAR;
-		else if (strcmp ( ComboArchiveType,"tar") == 0)
-			type = XARCHIVETYPE_TAR;
-		else if (strcmp ( ComboArchiveType,"tar.bz2") == 0)
-			type = XARCHIVETYPE_TAR_BZ2;
-		else if (strcmp ( ComboArchiveType,"tar.gz") == 0)
-			type = XARCHIVETYPE_TAR_GZ;
-		else if (strcmp ( ComboArchiveType,"tar.lzma") == 0)
-			type = XARCHIVETYPE_TAR_LZMA;
-		else if (strcmp ( ComboArchiveType,"tar.lzo") == 0)
-			type = XARCHIVETYPE_TAR_LZOP;
-		else if (strcmp ( ComboArchiveType,"jar") == 0 || strcmp ( ComboArchiveType,"zip") == 0 )
-			type = XARCHIVETYPE_ZIP;
-		else if (strcmp ( ComboArchiveType,"rpm") == 0)
-			type = XARCHIVETYPE_RPM;
-		else if (strcmp ( ComboArchiveType,"7z") == 0)
-			type = XARCHIVETYPE_7ZIP;
-		else if (strcmp ( ComboArchiveType,"lzh") == 0)
-			type = XARCHIVETYPE_LHA;
+		for (i = XARCHIVETYPE_FIRST; i < XARCHIVETYPE_TYPES; i++)
+		{
+			if (archiver[i].is_compressor)
+			{
+				GSList *list = archiver[i].type;
+
+				while (list)
+				{
+					if (list->data && (strcmp(ComboArchiveType, list->data) == 0))
+					{
+						type = i;
+						i = XARCHIVETYPE_TYPES;
+						break;
+					}
+
+					list = list->next;
+				}
+			}
+		}
+
+		gtk_widget_destroy(xa_file_chooser);
+
+		archive = xa_init_archive_structure(type);
+
+		archive->path[0] = g_strdup(my_path);
+		archive->path[1] = xa_escape_bad_chars(archive->path[0], ESCAPES);
+
+		g_free(my_path);
 
-		archive = xa_init_archive_structure (type);
-		archive->type = type;
-		gtk_widget_destroy (xa_file_chooser);
-		archive->path = g_strdup (my_path);
-		archive->escaped_path = xa_escape_bad_chars (archive->path , "$\'`\"\\!?* ()&|@#:;");
-		g_free (my_path);
 		return archive;
 	}
-	else if ( (response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT) )
-		gtk_widget_destroy (xa_file_chooser);
 
 	return NULL;
 }
