@@ -16,10 +16,22 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+#include <string.h>
 #include "string_utils.h"
+#include "utf8-fnmatch.h"
 
 #ifndef HAVE_MKDTEMP
-char *mkdtemp (gchar *tmpl)
+#include <errno.h>
+#include <glib/gstdio.h>
+#endif
+
+#ifndef HAVE_STRCASESTR
+#include <ctype.h>
+#endif
+
+#ifndef HAVE_MKDTEMP
+gchar *mkdtemp (gchar *tmpl)
 {
 	static const gchar LETTERS[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 	static guint64     value = 0;
@@ -80,15 +92,9 @@ const char *strcasestr(const char *haystack, const char *needle)
 	}
 	return NULL;
 }
-#endif /* !HAVE_STRCASESTR */
-
-gchar *xa_escape_bad_chars ( gchar *string , gchar *pattern)
-{
-	return xa_escape_common_chars (string, pattern, '\\', 0);
-}
-
-/* These functions are from File-Roller code */
+#endif
 
+/* This function is from File-Roller code */
 static int count_chars_to_escape (const char *str, const char *meta_chars)
 {
 	int meta_chars_n = strlen (meta_chars);
@@ -101,46 +107,76 @@ static int count_chars_to_escape (const char *str, const char *meta_chars)
 		for (i = 0; i < meta_chars_n; i++)
 			if (*s == meta_chars[i])
 			{
-        		n++;
-            	break;
+				n++;
+				break;
 			}
 		}
 	return n;
 }
-char *xa_escape_common_chars (const char *str, const char *meta_chars, const char  prefix, const char postfix)
+/* End code from File-Roller */
+
+static char *xa_escape_common_chars (const char *str, const char *meta_chars, const char  prefix, const char postfix)
 {
-        int         meta_chars_n = strlen (meta_chars);
-        char       *escaped;
-        int         i, new_l, extra_chars = 0;
-        const char *s;
-        char       *t;
+  int         meta_chars_n = strlen (meta_chars);
+  char       *escaped;
+  int         i, new_l, extra_chars = 0;
+  const char *s;
+  char       *t;
 
-        if (str == NULL)
-                return NULL;
+  if (str == NULL)
+          return NULL;
 
-        if (prefix)
-                extra_chars++;
-        if (postfix)
-                extra_chars++;
+  if (prefix)
+          extra_chars++;
+  if (postfix)
+          extra_chars++;
 
-        new_l = strlen (str) + (count_chars_to_escape (str, meta_chars) * extra_chars);
-        escaped = g_malloc0 (new_l + 1);
+  new_l = strlen (str) + (count_chars_to_escape (str, meta_chars) * extra_chars);
+  escaped = g_malloc0 (new_l + 1);
 
-        s = str;
-        t = escaped;
-        while (*s) {
-                gboolean is_bad = FALSE;
-                for (i = 0; (i < meta_chars_n) && !is_bad; i++)
-                        is_bad = (*s == meta_chars[i]);
-                if (is_bad && prefix)
-                        *t++ = prefix;
-                *t++ = *s++;
-                if (is_bad && postfix)
-                        *t++ = postfix;
-        }
-        *t = 0;
+  s = str;
+  t = escaped;
+  while (*s) {
+          gboolean is_bad = FALSE;
+          for (i = 0; (i < meta_chars_n) && !is_bad; i++)
+                  is_bad = (*s == meta_chars[i]);
+          if (is_bad && prefix)
+                  *t++ = prefix;
+          *t++ = *s++;
+          if (is_bad && postfix)
+                  *t++ = postfix;
+  }
+  *t = 0;
 
-        return escaped;
+  return escaped;
+}
+
+static gchar *xa_strip_current_working_dir_from_path (gchar *working_dir, gchar *filename)
+{
+	gchar *slash;
+	int len = 0;
+
+	if (working_dir == NULL)
+		return filename;
+	len = strlen(working_dir)+1;
+	slash = g_strrstr(filename,"/");
+	if (slash == NULL || ! g_path_is_absolute(filename))
+		return filename;
+
+	return filename+len;
+}
+
+static void xa_remove_slash_from_path (gchar *path)
+{
+	size_t len = strlen(path);
+
+	if (len > 0 && path[len - 1] == '/')
+		path[len - 1] = 0;
+}
+
+gchar *xa_escape_bad_chars (const gchar *string, const gchar *pattern)
+{
+	return xa_escape_common_chars (string, pattern, '\\', 0);
 }
 
 gchar *xa_remove_level_from_path (const gchar *path)
@@ -159,19 +195,6 @@ gchar *xa_remove_level_from_path (const gchar *path)
     return local_path;
 }
 
-gboolean file_extension_is (const char *filename, const char *ext)
-{
-	int filename_l, ext_l;
-
-	filename_l = strlen (filename);
-	ext_l = strlen (ext);
-
-    if (filename_l < ext_l)
-		return FALSE;
-    return strcasecmp (filename + filename_l - ext_l, ext) == 0;
-}
-/* End code from File-Roller */
-
 void xa_set_window_title (GtkWidget *window,gchar *title)
 {
 	gchar *x 	= NULL;
@@ -187,9 +210,9 @@ void xa_set_window_title (GtkWidget *window,gchar *title)
 		slash = title;
 
 	if (title == NULL)
-		x = g_strconcat ("Xarchiver ",VERSION,NULL);
+		x = g_strconcat(PACKAGE_NAME, " ", PACKAGE_VERSION, NULL);
 	else
-		x = g_strconcat (slash, " - Xarchiver ",VERSION,NULL);
+		x = g_strconcat(slash, " - ", PACKAGE_NAME, " ", PACKAGE_VERSION, NULL);
 	gtk_window_set_title (GTK_WINDOW (window),x);
 	g_free (x);
 }
@@ -215,87 +238,42 @@ gboolean match_patterns (char **patterns,const char *string,int flags)
 	return (result == 0);
 }
 
-gchar *xa_remove_path_from_archive_name(gchar *name)
+GSList *xa_collect_filenames (XArchive *archive, GSList *in)
 {
-	gchar *utf8_string,*text;
+	GSList *list = in, *out = NULL;
+	gchar *basename, *name;
 
-	text = g_strrstr (name,"/");
-	if (text != NULL)
+	while (list)
 	{
-		text++;
-		utf8_string = g_filename_display_name (text);
-	}
-	else
-		utf8_string = g_filename_display_name (name);
-
-	return utf8_string;
-}
-
-gchar *xa_escape_filename (gchar *filename,gchar *meta_chars)
-{
-	return xa_escape_common_chars (filename,meta_chars,'\\',0);
-}
-
-gchar *xa_strip_current_working_dir_from_path(gchar *working_dir,gchar *filename)
-{
-	gchar *slash;
-	int len = 0;
-
-	if (working_dir == NULL)
-		return filename;
-	len = strlen(working_dir)+1;
-	slash = g_strrstr(filename,"/");
-	if (slash == NULL || ! g_path_is_absolute(filename))
-		return filename;
-
-	return filename+len;
-}
-
-void xa_cat_filenames (XArchive *archive,GSList *list,GString *data)
-{
-	gchar *basename, *name, *e_filename;
-	GSList *slist = list;
-
-	while (slist)
-	{
-		if (archive->location_entry_path != NULL)
+		if (archive->location_path)
 		{
-			if (archive->full_path == 0)
+			if (archive->do_full_path)
 			{
-				basename = xa_strip_current_working_dir_from_path(archive->working_dir ? archive->working_dir : archive->tmp,slist->data);
-				name = g_strconcat(archive->location_entry_path,basename,NULL);
-				e_filename = xa_escape_filename(name,"$'`\"\\!?* ()[]&|:;<>#");
-				g_string_prepend (data,e_filename);
-				g_string_prepend_c (data,' ');
-				g_free(name);
+				name = g_strconcat(archive->location_path, list->data, NULL);
+				out = g_slist_append(out, name);
 			}
 			else
 			{
-				name = g_strconcat(archive->location_entry_path,slist->data,NULL);
-				e_filename = xa_escape_filename(name,"$'`\"\\!?* ()[]&|:;<>#");
-				g_string_prepend (data,e_filename);
-				g_string_prepend_c (data,' ');
-				g_free(name);
+				basename = xa_strip_current_working_dir_from_path(archive->child_dir ? archive->child_dir : archive->working_dir, list->data);
+				name = g_strconcat(archive->location_path, basename, NULL);
+				out = g_slist_append(out, name);
 			}
 		}
 		else
 		{
-			if (archive->full_path == 0)
-			{
-				basename = xa_strip_current_working_dir_from_path(archive->working_dir ? archive->working_dir : archive->tmp,slist->data);
-				e_filename = xa_escape_filename(basename,"$'`\"\\!?* ()[]&|:;<>#");
-				g_string_prepend (data,e_filename);
-				g_string_prepend_c (data,' ');
-			}
+			if (archive->do_full_path)
+				out = g_slist_append(out, g_strdup(list->data));
 			else
 			{
-				e_filename = xa_escape_filename(slist->data,"$'`\"\\!?* ()[]&|:;<>#");
-				g_string_prepend (data,e_filename);
-				g_string_prepend_c (data,' ');
+				basename = xa_strip_current_working_dir_from_path(archive->child_dir ? archive->child_dir : archive->working_dir, list->data);
+				out = g_slist_append(out, g_strdup(basename));
 			}
 		}
-		slist = slist->next;
+
+		list = list->next;
 	}
+
+	return out;
 }
 
 GSList *xa_slist_copy(GSList *list)
@@ -311,7 +289,7 @@ GSList *xa_slist_copy(GSList *list)
 	return g_slist_reverse(y);
 }
 
-void xa_recurse_local_directory(gchar *path,GSList **list,gboolean recurse,gint type)
+void xa_recurse_local_directory (gchar *path, GSList **list, gboolean recurse)
 {
 	DIR *dir;
 	struct dirent *dirlist;
@@ -320,7 +298,7 @@ void xa_recurse_local_directory(gchar *path,GSList **list,gboolean recurse,gint
 
 	dir = opendir(path);
 	is_dir = g_file_test(path,G_FILE_TEST_IS_DIR);
-	if (is_dir && type != XARCHIVETYPE_ARJ && type != XARCHIVETYPE_TAR && is_tar_compressed(type) == FALSE)
+	if (is_dir)
 		*list = g_slist_prepend(*list,g_strdup(path));
 	if (dir == NULL)
 	{
@@ -330,7 +308,7 @@ void xa_recurse_local_directory(gchar *path,GSList **list,gboolean recurse,gint
 			*list = g_slist_prepend(*list,basename);
 			return;
 		}
-	}	
+	}
 
 	while ((dirlist = readdir(dir)))
 	{
@@ -341,7 +319,64 @@ void xa_recurse_local_directory(gchar *path,GSList **list,gboolean recurse,gint
 		if ( ! is_dir)
 			*list = g_slist_prepend(*list,fullname);
 		if (recurse && is_dir)
-			xa_recurse_local_directory(fullname,list,recurse,type);
+			xa_recurse_local_directory(fullname, list, recurse);
 	}
 	closedir(dir);
 }
+
+GString *xa_quote_filenames (GSList *file_list, const gchar *escape, gboolean slash)
+{
+	GString *files;
+	GSList *list;
+
+	files = g_string_new("");
+	list= file_list;
+
+	while (list)
+	{
+		gchar *shellname, *escaped = NULL;
+
+		if (!slash)
+			xa_remove_slash_from_path(list->data);
+
+		if (escape)
+			escaped = xa_escape_bad_chars(list->data, "\\");
+
+		shellname = g_shell_quote(escaped ? escaped : list->data);
+
+		g_free(escaped);
+
+		if (escape)
+			escaped = xa_escape_bad_chars(shellname, escape);
+
+		g_string_prepend(files, escaped ? escaped : shellname);
+		g_string_prepend_c(files, ' ');
+
+		g_free(escaped);
+		g_free(shellname);
+
+		list = list->next;
+	}
+
+	g_slist_foreach(file_list, (GFunc) g_free, NULL);
+	g_slist_free(file_list);
+
+	return files;
+}
+
+gchar *xa_quote_shell_command (const gchar *string, gboolean unescaped)
+{
+	gchar *unquoted = NULL, *quoted, *escaped;
+
+	if (!unescaped)
+		unquoted = g_shell_unquote(string, NULL);
+
+	quoted = g_shell_quote(unquoted ? unquoted : string);
+
+	escaped = xa_escape_bad_chars(quoted, "\"");
+
+	g_free(quoted);
+	g_free(unquoted);
+
+	return escaped;
+}
