@@ -16,40 +16,35 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef STRING_UTILS_H
-#define STRING_UTILS_H
+#ifndef XARCHIVER_STRING_UTILS_H
+#define XARCHIVER_STRING_UTILS_H
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
-
 #include <gtk/gtk.h>
-#include <glib.h>
-#include <string.h>
-#include <dirent.h>
-#include "string_utils.h"
-#include "utf8-fnmatch.h"
-#include "errno.h"
 #include "archive.h"
-#include "tar.h"
+
+#define ESCAPES " \"'\\"
+
+#ifndef HAVE_MKDTEMP
+gchar *mkdtemp(gchar *);
+#else
+#include <stdlib.h>
+#endif
 
 #ifndef HAVE_STRCASESTR
 const char *strcasestr(const char *haystack, const char *needle);
-#endif
-#ifndef HAVE_MKDTEMP
-char *mkdtemp (gchar *tmpl);
-#endif
-gchar *xa_escape_bad_chars ( gchar *string , gchar *pattern);
-char *xa_escape_common_chars (const char *str, const char *meta_chars, const char  prefix, const char  postfix);
-gchar *xa_remove_level_from_path (const gchar *path);
-gboolean file_extension_is (const char *filename, const char *ext);
-void xa_set_window_title ( GtkWidget *window , gchar *title);
-gboolean match_patterns (char **patterns,const char *string,int flags);
-gchar *xa_remove_path_from_archive_name(gchar *name);
-void xa_cat_filenames (XArchive *,GSList *,GString *);
-gchar *xa_escape_filename (gchar *filename,gchar *meta_chars);
-GSList *xa_slist_copy(GSList *);
-void xa_recurse_local_directory(gchar *,GSList **,gboolean,gint);
-gchar *xa_strip_current_working_dir_from_path(gchar *,gchar *);
+#else
+#include <string.h>
 #endif
 
+gboolean match_patterns(char **, const char *, int);
+gchar *xa_escape_bad_chars(const gchar *, const gchar *);
+GSList *xa_collect_filenames(XArchive *, GSList *);
+GString *xa_quote_filenames(GSList *, const gchar *, gboolean);
+gchar *xa_quote_shell_command(const gchar *, gboolean);
+void xa_recurse_local_directory(gchar *, GSList **, gboolean);
+gchar *xa_remove_level_from_path(const gchar *);
+void xa_set_window_title(GtkWidget *, gchar *);
+GSList *xa_slist_copy(GSList *);
+
+#endif
